-- reg
--
-- reg modelliert ein Register mit variabler Breite mit
-- Freigabeeingang und asynchronem Setz- und Ruecksetzeingang
-- (schaltet bei positiver Taktflanke)

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;

entity reg is
	GENERIC (WIDTH: positive  := 8);
	port( d		: in	std_logic_vector(WIDTH-1 DOWNTO 0);	-- Eingang
			clk	: in	std_logic;	-- Takt
			ena	: in	std_logic := '1';	-- Freigabe-Eingang
			pr_n	: in	std_logic := '1';	-- asynchroner Setzeingang
			clr_n	: in	std_logic := '1';	-- asynchroner Setzeingang
			q		: out	std_logic_vector(WIDTH-1 DOWNTO 0);	-- Ausgang
			q_n	: out	std_logic_vector(WIDTH-1 DOWNTO 0)); -- negierter Ausgang
end entity reg;


architecture structure1 of reg is
begin
	process(d, clk, ena, pr_n, clr_n) is
		variable FlipFlop : std_logic_vector(WIDTH-1 DOWNTO 0); -- Speicherinhalt
	begin

		assert not (pr_n='0' and clr_n='0') report "Sie haben versucht, das Register geleichzeitig zu setzten (pr_n='0') und rueckzusetzen (clr_n='0') !" severity warning;

		if clr_n='0' then
			FlipFlop := (others => '0');
		elsif pr_n='0' then
			FlipFlop := (others => '1');
		elsif clk='1' and clk'event and ena='1' then
			FlipFlop := d;
		end if; 	
		q <= FlipFlop;
		q_n <= not FlipFlop;
	end process;
end architecture structure1;
